/*********************************************************************NVMH3****
File:  $Id: //sw/devtools/SDK/9.5/SDK/MEDIA/HLSL/scene_anaglyphic.fx#3 $

Copyright NVIDIA Corporation 2002-2004
TO THE MAXIMUM EXTENT PERMITTED BY APPLICABLE LAW, THIS SOFTWARE IS PROVIDED
*AS IS* AND NVIDIA AND ITS SUPPLIERS DISCLAIM ALL WARRANTIES, EITHER EXPRESS
OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, IMPLIED WARRANTIES OF MERCHANTABILITY
AND FITNESS FOR A PARTICULAR PURPOSE.  IN NO EVENT SHALL NVIDIA OR ITS SUPPLIERS
BE LIABLE FOR ANY SPECIAL, INCIDENTAL, INDIRECT, OR CONSEQUENTIAL DAMAGES
WHATSOEVER (INCLUDING, WITHOUT LIMITATION, DAMAGES FOR LOSS OF BUSINESS PROFITS,
BUSINESS INTERRUPTION, LOSS OF BUSINESS INFORMATION, OR ANY OTHER PECUNIARY LOSS)
ARISING OUT OF THE USE OF OR INABILITY TO USE THIS SOFTWARE, EVEN IF NVIDIA HAS
BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.


Comments:
    Gooch-style diffuse texturing, calculated per-vertex.
	Untextured and textured techniques are provided.

******************************************************************************/

float Script : STANDARDSGLOBAL <
    string UIWidget = "none";
    string ScriptClass = "scene";
    string ScriptOrder = "standard";
    string ScriptOutput = "color";
    string Script = "Technique=stereo;";
> = 0.8;

#include <include\\Quad.fxh>
#include <include\\nvMatrix.fxh>

/************* UNTWEAKABLES **************/

float4x4 WorldITXf : WorldInverseTranspose < string UIWidget="None"; >;
float4x4 WorldViewXf : WorldView < string UIWidget="None"; >;
float4x4 ProjXf : Projection < string UIWidget="None"; >;
float4x4 WorldXf : World < string UIWidget="None"; >;

float4 ClearColor <
	// string UIWidget = "color";
	string UIWidget = "none";
	string UIName = "Scene Background";
> = {0.0,0.0,0.0,0.0};

float ClearDepth <string UIWidget = "none";> = 1.0;

/************* TWEAKABLES ****************/

float Shift <
	string UIName = "Eye Shift";
	string UIWidget = "slider";
	float UIMin = -0.3;
	float UIMax = 0.3;
	float UIStep = 0.001;
> = 0.07f;

float4 MaskColorL <
	string UIWidget="color";
	string UIName = "Left Lens Color";
> = {1,0,0,1};

float4 MaskColorR <
	string UIWidget="color";
	string UIName = "Right Lens Color";
> = {0,1,0,1};

float4 LightPos : Position
<
	string Object = "PointLight";
	string UIName = "Lamp Position";
	string Space = "World";
> = {100.0f, 100.0f, 100.0f, 0.0f};

/******** RTT ****************************/

DECLARE_QUAD_TEX(LMap,LSampler,"A8B8G8R8")
DECLARE_QUAD_TEX(RMap,RSampler,"A8B8G8R8")
DECLARE_QUAD_DEPTH_BUFFER(DepthMap,"D24S8")

/************* DATA STRUCTS **************/

/* data from application vertex buffer */
struct appdata {
    float3 Position	: POSITION;
    float4 UV		: TEXCOORD0;
    float4 Normal	: NORMAL;
};

/* data passed from vertex shader to pixel shader */
struct vertexOutput {
    float4 HPosition	: POSITION;
    float4 TexCoord0	: TEXCOORD0;
    float4 diffCol	: COLOR0;
};

/*********** vertex shader ******/

vertexOutput goochVS(appdata IN,uniform float4x4 WorldViewProjXf)
{
    vertexOutput OUT;
    float3 Nn = normalize(mul(IN.Normal, WorldITXf).xyz);
    float4 Po = float4(IN.Position.xyz,1);
    //compute worldspace position
    float3 Pw = mul(Po, WorldXf).xyz;
    float3 Ln = normalize(LightPos - Pw);
    float mixer = 0.5 * (dot(Ln,Nn) + 1.0);
    OUT.diffCol = float4(mixer.xxx,1);
    OUT.TexCoord0 = IN.UV;
    OUT.HPosition = mul(Po, WorldViewProjXf);
    return OUT;
}

/********* pixel shader ********/

float4 goochPS(vertexOutput IN,uniform float4 Mask) : COLOR {
    return (IN.diffCol * Mask);
}

QUAD_REAL4 mixPS(QuadVertexOutput IN) : COLOR
{   
	QUAD_REAL4 R = tex2D(RSampler, IN.UV);
	QUAD_REAL4 L = tex2D(LSampler, IN.UV);
	return float4((R.xyz+L.xyz),R.w);
}  

/*** TECHNIQUES **********/

technique Stereo <
	string Script =
					"Pass=L;"
					"Pass=R;"
					"Pass=Mix;";
> {
    pass L  <
		string Script =
					"RenderColorTarget=LMap;"
					"RenderDepthStencilTarget=DepthMap;"
					"ClearSetColor=ClearColor;"
					"ClearSetDepth=ClearDepth;"
					"Clear=Color;"
					"Clear=Depth;"
					"Draw=geometry;";
    > {		
	    VertexShader = compile vs_1_1 goochVS(
						mul(mul(WorldViewXf,nvTranslateXf(Shift,0,0)),ProjXf));
	    ZEnable = true;
	    ZWriteEnable = true;
	    CullMode = None;
        PixelShader = compile ps_2_a goochPS(MaskColorL);
    }
    pass R  <
		string Script =
					"RenderColorTarget=RMap;"
					"RenderDepthStencilTarget=DepthMap;"
					"ClearSetColor=ClearColor;"
					"ClearSetDepth=ClearDepth;"
					"Clear=Color;"
					"Clear=Depth;"
					"Draw=geometry;";
    > {		
	    VertexShader = compile vs_1_1 goochVS(
						mul(mul(WorldViewXf,nvTranslateXf(-Shift,0,0)),ProjXf));
	    ZEnable = true;
	    ZWriteEnable = true;
	    CullMode = None;
        PixelShader = compile ps_2_a goochPS(MaskColorR);
    }
	pass Mix <
		string Script = "RenderColorTarget=;"
						"RenderDepthStencilTarget=;"
						// "ClearSetColor=ClearColor;"
						// "ClearSetDepth=ClearDepth;"
						// "Clear=Color;"
						// "Clear=Depth;"
						"Draw=Buffer;";        	
	> {
		VertexShader = compile vs_2_0 ScreenQuadVS();
		cullmode = none;
		ZEnable = false;
		ZWriteEnable = false;
		PixelShader  = compile ps_2_a mixPS();
	}
}

/***************************** eof ***/
